
using System;
using System.Collections.Generic;

public interface IObserver
{
    void Update(string message);
}

public interface ISubject
{
    void Attach(IObserver observer);
    void Detach(IObserver observer);
    void Notify(string message);
}

public class CasinoGame : ISubject
{
    private List<IObserver> observers = new List<IObserver>();

    public void Attach(IObserver observer)
    {
        observers.Add(observer);
    }

    public void Detach(IObserver observer)
    {
        observers.Remove(observer);
    }

    public void Notify(string message)
    {
        foreach (var observer in observers)
        {
            observer.Update(message);
        }
    }
}

public class PlayerObserver : IObserver
{
    public void Update(string message)
    {
        Console.WriteLine($"[Observer] {message}");
    }
}

public class BalanceManager
{
    private static BalanceManager _instance;
    private int balance;
    public const int InitialBalance = 1000;

    private BalanceManager()
    {
        balance = InitialBalance;
    }

    public static BalanceManager Instance => _instance ??= new BalanceManager();

    public int Balance
    {
        get => balance;
        set => balance = value;
    }
}

public class Casino
{
    public const int MaxRouletteBetAmount = 1000;
    public const int BlackjackMaxBetAmount = 1000;
    public const int DealerStandThreshold = 17;
    public const int BlackjackBustValue = 21;

    public CasinoGame casinoGame = new CasinoGame();
    public PlayerObserver playerObserver = new PlayerObserver();

    public Casino()
    {
        casinoGame.Attach(playerObserver);
    }

    public void Start()
    {
        while (true)
        {
            Console.Clear();
            ShowMainMenu();

            string choice = Console.ReadLine();
            if (choice == "1")
            {
                PlayRoulette();
            }
            else if (choice == "2")
            {
                PlayBlackjack();
            }
            else if (choice == "3")
            {
                break;
            }
        }
    }

    public void ShowMainMenu()
    {
        Console.WriteLine("Willkommen im Casino!");
        Console.WriteLine("1. Roulette spielen");
        Console.WriteLine("2. Blackjack spielen");
        Console.WriteLine("3. Beenden");
        Console.Write("Wähle eine Option: ");
    }

    public int GetBetAmount()
    {
        while (true)
        {
            Console.Write("Setze deinen Einsatz: ");
            if (int.TryParse(Console.ReadLine(), out int bet) && bet > 0 && bet <= BalanceManager.Instance.Balance)
            {
                return bet;
            }
            Console.WriteLine("Ungültiger Einsatz. Versuche es erneut.");
        }
    }

    public void PlayRoulette()
    {
        while (true)
        {
            Console.Clear();
            Console.WriteLine("Roulette Spiel");
            Console.WriteLine($"Dein Guthaben: {BalanceManager.Instance.Balance} Chips");

            int betAmount = GetBetAmount();
            string betType = GetBetType();

            int winningNumber = SpinRouletteWheel();
            Console.WriteLine($"Die Kugel fällt auf: {winningNumber}");

            if (IsWinningBet(betType, winningNumber))
            {
                int winnings = CalculateWinnings(betType, betAmount);
                Console.WriteLine($"Du hast gewonnen! Gewinn: {winnings} Chips");
                BalanceManager.Instance.Balance += winnings;
                casinoGame.Notify($"Spieler hat {winnings} Chips im Roulette gewonnen.");
            }
            else
            {
                Console.WriteLine("Leider verloren.");
                BalanceManager.Instance.Balance -= betAmount;
                casinoGame.Notify($"Spieler hat {betAmount} Chips im Roulette verloren.");
            }

            Console.WriteLine($"Neues Guthaben: {BalanceManager.Instance.Balance} Chips");
            Console.WriteLine("Drücke Enter für eine neue Runde oder 'q' für das Menü");
            if (Console.ReadLine().ToLower() == "q") break;
        }
    }

    public string GetBetType()
    {
        while (true)
        {
            Console.Write("Setze auf 'rot', 'schwarz', '1-12', '13-24', '25-36', 'top-row', 'middle-row', 'bottom-row', 'gerade', 'ungerade', '1-18', '19-36' oder eine Zahl (0-36): ");
            string bet = Console.ReadLine().ToLower();
            if (IsValidRouletteBet(bet))
            {
                return bet;
            }
            Console.WriteLine("Ungültige Wette. Versuche es erneut.");
        }
    }

    public bool IsValidRouletteBet(string bet)
    {
        return bet == "rot" || bet == "schwarz" || bet == "1-12" || bet == "13-24" || bet == "25-36"
            || bet == "top-row" || bet == "middle-row" || bet == "bottom-row" || bet == "gerade"
            || bet == "ungerade" || bet == "1-18" || bet == "19-36" || int.TryParse(bet, out int _);
    }

    public int SpinRouletteWheel()
    {
        return new Random().Next(0, 37);
    }

    public bool IsWinningBet(string bet, int result)
    {
        if (bet == "rot" && IsRed(result)) return true;
        if (bet == "schwarz" && !IsRed(result) && result != 0) return true;
        if (bet == "1-12" && result >= 1 && result <= 12) return true;
        if (bet == "13-24" && result >= 13 && result <= 24) return true;
        if (bet == "25-36" && result >= 25 && result <= 36) return true;
        if (bet == "gerade" && result % 2 == 0 && result != 0) return true;
        if (bet == "ungerade" && result % 2 == 1) return true;
        if (bet == "1-18" && result >= 1 && result <= 18) return true;
        if (bet == "19-36" && result >= 19 && result <= 36) return true;
        if (int.TryParse(bet, out int number) && number == result) return true;
        return false;
    }

    public int CalculateWinnings(string bet, int betAmount)
    {
        if (int.TryParse(bet, out int num) && num == 0) return betAmount * 36;
        if (int.TryParse(bet, out _)) return betAmount * 35;
        if (bet == "1-12" || bet == "13-24" || bet == "25-36") return betAmount * 3;
        return betAmount * 2;
    }

    public bool IsRed(int number)
    {
        int[] redNumbers = { 1, 3, 5, 7, 9, 12, 14, 16, 18, 19, 21, 23, 25, 27, 30, 32, 34, 36 };
        return Array.Exists(redNumbers, n => n == number);
    }

    public void PlayBlackjack()
    {
        while (true)
        {
            Console.Clear();
            Console.WriteLine("Blackjack Spiel");
            Console.WriteLine($"Dein Guthaben: {BalanceManager.Instance.Balance} Chips");

            int betAmount = GetBetAmount();
            List<int> playerHand = new List<int> { DrawCard(), DrawCard() };
            List<int> dealerHand = new List<int> { DrawCard() };

            Console.WriteLine($"Deine Karten: {string.Join(", ", playerHand)} (Wert: {GetHandValue(playerHand)})");
            Console.WriteLine($"Dealer zeigt: {dealerHand[0]}");

            // Spielerzug
            while (GetHandValue(playerHand) < BlackjackBustValue)
            {
                Console.Write("Hit (h) oder Stand (s)? ");
                string choice = Console.ReadLine().ToLower();

                if (choice == "h")
                {
                    playerHand.Add(DrawCard());
                    Console.WriteLine($"Neue Karten: {string.Join(", ", playerHand)} (Wert: {GetHandValue(playerHand)})");
                }
                else if (choice == "s")
                {
                    break;
                }
            }

            int playerValue = GetHandValue(playerHand);
            if (playerValue > BlackjackBustValue)
            {
                Console.WriteLine("Du hast überzogen! Verloren.");
                BalanceManager.Instance.Balance -= betAmount;
                casinoGame.Notify($"Spieler hat {betAmount} Chips im Blackjack verloren.");
            }
            else
            {
                // Dealer spielt
                dealerHand.Add(DrawCard());
                DealerPlay(ref dealerHand);
                int dealerValue = GetHandValue(dealerHand);

                Console.WriteLine($"Dealer hat: {string.Join(", ", dealerHand)} (Wert: {dealerValue})");

                if (dealerValue > BlackjackBustValue || playerValue > dealerValue)
                {
                    Console.WriteLine("Du hast gewonnen!");
                    BalanceManager.Instance.Balance += betAmount;
                    casinoGame.Notify($"Spieler hat {betAmount} Chips im Blackjack gewonnen.");
                }
                else if (playerValue < dealerValue)
                {
                    Console.WriteLine("Der Dealer gewinnt.");
                    BalanceManager.Instance.Balance -= betAmount;
                    casinoGame.Notify($"Spieler hat {betAmount} Chips im Blackjack verloren.");
                }
                else
                {
                    Console.WriteLine("Unentschieden!");
                }
            }

            Console.WriteLine($"Neues Guthaben: {BalanceManager.Instance.Balance} Chips");
            Console.WriteLine("Drücke Enter für eine neue Runde oder 'q' für das Menü.");
            if (Console.ReadLine().ToLower() == "q") break;
        }
    }

    public void DealerPlay(ref List<int> dealerHand)
    {
        while (GetHandValue(dealerHand) < DealerStandThreshold)
        {
            dealerHand.Add(DrawCard());
        }
    }

    public int DrawCard()
    {
        return new Random().Next(1, 11);
    }

    public int GetHandValue(List<int> hand)
    {
        int total = 0;
        int aces = 0;

        foreach (int card in hand)
        {
            if (card == 1)
            {
                aces++;
                total += 11;
            }
            else
            {
                total += card;
            }
        }

        while (total > BlackjackBustValue && aces > 0)
        {
            total -= 10;
            aces--;
        }

        return total;
    }
}

public class Program
{
    public static void Main()
    {
        Casino casino = new Casino();
        casino.Start();
    }
}